line1=Opcje konfigurowalne,11
show_list=Wyświetlanie serwerów wirtualnych jako,1,0-Ikony,1-Listę
show_order=Kolejność serwerów wirtualnych wg,1,0-Porządku w&nbsp;plikach konfiguracyjnych,1-Nazwy servera,2-Adresu IP
max_servers=Maksymalna liczba wyświetlanych serwerów,0,5
virt_file=Serwery wirtualne dodawane w&nbsp;pliku,3,httpd.conf
virt_name=Wzór nazwy pliku dla serwerów wirtualnych,3,Domyślny ($DOM.conf)
link_dir=Katalog do tworzenia łączy dla nowych serwerów wirtualnych,3,Brak
test_config=Sprawdzać plik konfiguracyjny przed zastosowaniem zmian?,1,1-Tak,0-Nie
test_manual=Testować plik konfiguracyjny po ręcznych zmianach?,1,1-Tak,0-Nie
test_always=Testować plik konfiguracji po innych zmianach?,1,1-Tak,0-Nie
test_apachectl=Testuj konfigurację z komendami,1,1-<tt>apachectl configtest</tt>,0-<tt>httpd</tt> z opcją <tt>-D</tt>
show_names=Wyświetlać nazwy dyrektyw Apache?,1,1-Tak,0-Nie
apache_docbase=Katalog bazowy dla dokumentacji Apache,3,witryna Apache
auto_mods=Wykrywać automatycznie moduły Apache?,1,1-Tak,0-Nie
allow_virtualmin=Zezwalać na edycję domen zarządzanych przez Virtualmin?,1,1-Tak,0-Nie
line2=Konfiguracja systemu,11
httpd_dir=Katalog główny serwera Apache,0
httpd_path=Ścieżka do pliku programu httpd,0
httpd_version=Wersja Apache,3,Automatycznie
apachectl_path=Ścieżka do polecenia apachectl,3,Brak
start_cmd=Polecenie uruchamiające apache'a,3,Automatycznie
stop_cmd=Polecenie zatrzymujące apache'a,3,Automatycznie
apply_cmd=Komenda do zastosowania konfiguracji,10,-Użyj apachectl lub signal HUP,Restart-Uruchom komendy stop i strat
graceful_cmd=Komenda do ponownego odczytu konfiguracji,3,Taka sama jak komenda zastosowania
httpd_conf=Ścieżka do httpd.conf,3,Automatycznie
srm_conf=Ścieżka do srm.conf,3,Automatycznie
access_conf=Ścieżka do access.conf,3,Automatycznie
mime_types=Ścieżka do mime.types,3,Automatycznie
pid_file=Ścieżka do pliku PID Apache,3,Automatycznie
line3=Zmienne Apache,11
defines_file=Plik środowiska definiujący zmienne Apache,3,Brak
defines_name=Nazwa zmiennej środowiskowej w powyższym pliku,0
defines_mods=Katalogi zawierające moduły dla zmiennych HAVE_,9,40,3,\t
