/**
 * @license Copyright (c) 2003-2023, CKSource Holding sp. z o.o. All rights reserved.
 * For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license/
 */

/**
 * This file was added automatically by CKEditor builder.
 * You may re-use it at any time to build CKEditor again.
 *
 * If you would like to build CKEditor online again
 * (for example to upgrade), visit one the following links:
 *
 * (1) https://ckeditor.com/cke4/builder
 *     Visit online builder to build CKEditor from scratch.
 *
 * (2) https://ckeditor.com/cke4/builder/802eb6af7b2e39f02517321346a376a7
 *     Visit online builder to build CKEditor, starting with the same setup as before.
 *
 * (3) https://ckeditor.com/cke4/builder/download/802eb6af7b2e39f02517321346a376a7
 *     Straight download link to the latest version of CKEditor (Optimized) with the same setup as before.
 *
 * NOTE:
 *    This file is not used by CKEditor, you may remove it.
 *    Changing this file will not change your CKEditor configuration.
 */

var CKBUILDER_CONFIG = {
	skin: 'moono-lisa',
	preset: 'full',
	ignore: [
		'.DS_Store',
		'.bender',
		'.editorconfig',
		'.gitattributes',
		'.gitignore',
		'.idea',
		'.jscsrc',
		'.jshintignore',
		'.jshintrc',
		'.mailmap',
		'.npm',
		'.nvmrc',
		'.travis.yml',
		'README.md',
		'bender-err.log',
		'bender-out.log',
		'bender.ci.js',
		'bender.js',
		'dev',
		'gruntfile.js',
		'less',
		'node_modules',
		'package-lock.json',
		'package.json',
		'tests'
	],
	plugins : {
		'base64image' : 1,
		'basicstyles' : 1,
		'bidi' : 1,
		'blockquote' : 1,
		'colorbutton' : 1,
		'colordialog' : 1,
		'contextmenu' : 1,
		'copyformatting' : 1,
		'dialogadvtab' : 1,
		'div' : 1,
		'elementspath' : 1,
		'enterkey' : 1,
		'entities' : 1,
		'find' : 1,
		'floatingspace' : 1,
		'font' : 1,
		'format' : 1,
		'forms' : 1,
		'horizontalrule' : 1,
		'htmlwriter' : 1,
		'iframe' : 1,
		'imageresizerowandcolumn' : 1,
		'indentblock' : 1,
		'indentlist' : 1,
		'insertpre' : 1,
		'justify' : 1,
		'link' : 1,
		'list' : 1,
		'liststyle' : 1,
		'magicline' : 1,
		'openlink' : 1,
		'pagebreak' : 1,
		'preview' : 1,
		'print' : 1,
		'removeformat' : 1,
		'showblocks' : 1,
		'showborders' : 1,
		'sourcearea' : 1,
		'tab' : 1,
		'table' : 1,
		'tableresizerowandcolumn' : 1,
		'tabletools' : 1,
		'toolbar' : 1,
		'undo' : 1,
		'wysiwygarea' : 1
	},
	languages : {
		'af' : 1,
		'ar' : 1,
		'bg' : 1,
		'ca' : 1,
		'cs' : 1,
		'da' : 1,
		'de' : 1,
		'el' : 1,
		'en' : 1,
		'es' : 1,
		'fa' : 1,
		'fi' : 1,
		'fr' : 1,
		'he' : 1,
		'hr' : 1,
		'id' : 1,
		'it' : 1,
		'ja' : 1,
		'ko' : 1,
		'lt' : 1,
		'lv' : 1,
		'ms' : 1,
		'nl' : 1,
		'no' : 1,
		'pl' : 1,
		'pt' : 1,
		'pt-br' : 1,
		'ro' : 1,
		'ru' : 1,
		'sk' : 1,
		'sl' : 1,
		'sq' : 1,
		'sv' : 1,
		'th' : 1,
		'tr' : 1,
		'uk' : 1,
		'vi' : 1,
		'zh' : 1,
		'zh-cn' : 1
	}
};