line0=Konfigurační soubory a programy,11
local_cf=Konfigurační soubor nebo adresář SpamAssassinu,0
spamassassin=Úplná cesta pro příkaz SpamAssassin,0
sa_learn=Úplná cesta pro příkaz sa-learn,0
warn_procmail=Zobrazit varování v případě&#44; kdy není SpamAssassin nastaven?,1,1-Ano,0-Ne
procmailrc=Konfigurační soubor Procmailu,3,Z modulu Procmailu
processes=Názvy procesu SpamAssassin démona,0
restart_cmd=Příkaz pro restartování procesů,3,Ihned poslat HUP signál
procmail_cmd=Příkaz SpamAssassinu pro konfiguraci Procmailu,10,*-Detekovat automaticky
before_cmd=Příkaz spuštěný před změnou pravidel,3,Nic
after_cmd=Příkaz spuštěný po změně pravidel,3,Nic
line1=Volby pro LDAP a SQL server,11
mode=Uložit nastavení SpamAssassinu do,1,0-Konfiguračních souborů,1-MySQL databáze,2-PostgreSQL databáze,3-LDAP databáze
server=Hostname LDAP nebo SQL serveru,0
port=Port LDAP nebo SQL serveru,3,Výchozí
user=Přihlášení k LDAP nebo SQL serveru,0
pass=Heslo pro LDAP nebo SQL server,0
db=Název SQL databáze,3,Není potřeba
base=LDAP základna DN,3,Není potřeba
dbglobal=Tag pro globální nastavení v SQL databází,3,Výchozí (<tt>@GLOBAL</tt>)
attr=LDAP atribut pro SpamAssassin preference,3,Výchozí (<tt>spamassassin</tt>)
uid=LDAP atribut pro uživatelská jména,3,Výchozí (<tt>uid</tt>)
addto=Přidat nové direktivy do,1,1-SQL nebo LDAP databáze,0-Konfiguračního souboru
