line1=Opcje konfiguracyjne,11
expert=Pozwalać na wybór poszczególnych poziomów pracy systemu,1,1-Tak,0-Nie
desc=Wyświetlać działania wraz z&nbsp;opisami,1,2,Tak&#44; i&nbsp;pokazuj wszystkie poziomy,1-Tak,0-Nie
order=Wyświetlać kolejność bootowania akcji?,1,1-Tak,0-Nie
status_check=Pokazywać bieżący stan akcji,1,2-Na stronach indeksowej i&nbsp;akcji,1-Tylko na stronie akcji,0-Nie
sort_mode=Sortuj akcje według,1,1-Kolejności bootowania,0-Nazwy
line2=Konfiguracja systemu,11
init_base=Katalog zawierający podkatalogi poziomów pracy,0
init_dir=Katalog zawierający nadrzędne skrypty startowe,0
order_digits=Liczba cyfr określających kolejność uruchamiania,0
local_script=Lokalny skrypt poleceń startowych,3
local_down=Lokalny skrypt poleceń wyłączania,3,Brak
reboot_command=Polecenie restartujące system,0
shutdown_command=Polecenie zamykające system,0
inittab_id=ID inicjalnego poziomu pracy w&nbsp;inittab-ie,0
rc_dir=Katalogi skryptów rc FreeBSD,3,Brak
rc_conf=Plik konfiguracyjny FreeBSD,3,Brak
line3=Konfiguracja systemu OSX,11
startup_dirs=Katalogi StartupItems Darwina,0
darwin_setup=Katalog dla indywidualnych StartupItems,0
hostconfig=Plik hostconfig Darwina,0
plist=Nazwa p-listy w katalogu StartupItems,0
