line0=Nastavitelné volby,11
view_condition=Zobrazit podmínku v seznamu pravidel?,1,1-Ano,0-Ne
view_comment=Zobrazit komentář v seznamu pravidel?,1,1-Ano,0-Ne
comment_mod=Uchovávat komentáře jako,1,0-# komentáře v uloženém souboru,1-&#45;&#45;vylby komnetáře
cluster_mode=Aktualizovat sborník serverů,1.0-Kdykoliv při změně,1-Pouze při uplatnění konfigurace
before_cmd=Příkaz&#44; který má být spuštěn před změnou pravidel,3,Nic
after_cmd=Příkaz&#44; který má být spuštěn po změně pravidel,3,Nic
before_apply_cmd=Příkaz&#44; který má být spuštěn před uplatněním konfigurace,3,Nic
after_apply_cmd=Příkaz&#44; který má být spuštěn po uplatnění konfigurace,3,Nic
save_file=File to save/edit <tt>IPv4</tt> rules,3,Use operating system or Webmin default
direct=Přímo editovat pravidla firewallu namísto ukládání souboru?,1,1-Ano,0-Ne
save_file6=File to save/edit <tt>IPv6</tt> rules,3,Use operating system or Webmin default
