body
{
  font-size: 12px;
}
#list_form {
  clear: both;
}
#listing {
  border-collapse: collapse;
  width: 100%;
}
#listing td {
  padding: 1px;
}
#listing tr.header {
  font-weight: bold;
  background: #ddd;
}

#list_form tr.checked {
  background: #bfb;
}
#list_form tr.checked.even {
  background: #cfc;
}
#list_form tr.checked:hover {
  background: #bdb;
}
a {
  color: #000;
}
a.active {
    font-weight: bold;
}
input.hidden {
  visibility: hidden;
  margin: 0;
}
tr.even {
  background: #eee;
}
td.right {
  text-align: right;
}
#quicks {
    float: right;
}
#quicks a {
    display: inline-block;
}
#quicks a:hover {
    background: #ccc;
}

.ui_columns {
    clear: both;
    width: 100%;
}

#info {
    float: right;
    color: #d22;
    margin-right: 10px;
}

.dataTables_filter input {
    margin-top: 10px !important;
}

table.dataTable thead .sorting::after,
table.dataTable thead .sorting_asc::after,
table.dataTable thead .sorting_desc::after {
    content: "" !important;
}

.dataTables_wrapper .dataTables_paginate .paginate_button {
    border: none;
    min-width: none;
    margin: 0;
    padding: 0;
}

.dataTables_wrapper .dataTables_paginate .paginate_button:active,
.dataTables_wrapper .dataTables_paginate .paginate_button:hover {
    background: none;
    box-shadow: none;
    border: none;
}

.breadcrumb {
    border: solid 1px #ddd;
    background: #fbfbfb;
}
.breadcrumb i {
    font-size: 16px;
}

.hdd-icon {
    position: relative;
    top: 6px;
}

#chattrForm input,
#chconForm input,
#chownForm input {
    width: 200px;
}

.modal {
    display:none;
}

textarea {
    font-family: consolas, monospace !important;
}

label + select, label + input[type="text"] {
    max-width: 100%;
}

select, input[type="button"][onclick], button.btn-default.file_chooser_button, button.btn-default.file_chooser_button_emulate, input.form-control.ui_textbox, td > select + input, input + input, input + select, select + input, select + select, input[type="submit"] + select, input[type="submit"] + input[type="submit"], div.table-responsive + input, div.table-responsive > select + input, div.table-responsive > input + select, div.table-responsive > input {
    margin: 0 !important;
}

.action-link {
  padding-right: 3px;
}

.btn:not(.btn-xs):not(.btn-tiny):not(.ui_link_replaced), .csf button.input, .csf input[type=submit], input[type=button], input[type=reset], input[type=submit], button.btn {
    padding: 6px 8px;
}

a:hover {
 cursor:pointer;
}
/* by https://github.com/iliajie */
#list_form tbody tr.checked:not(.m-active-o):not(.m-not-active):hover,
#list_form tbody tr:hover:not(.m-active-o):not(.m-not-active) {
    background: #d3e4f4 !important;
}

a.pages {
    padding: 0 3px;
}

.breadcrumb {
    margin-bottom: 10px;
}

.total {
    clear: both;
    margin-bottom: 10px;
}

span[data-acls],
span[data-attributes],
span[data-secontext] {
  font-family: monospace;
  font-size: 11px;
}
