cal_max=Maksymalna liczba linii logu przekazywanych do <tt>calamaris</tt>,3,Nieograniczona
crypt_conf=Metoda szyfrowania haseł dla proxy,1,0-crypt,1-md5base64
sort_conf=Uporządkować użytkowników proxy,1,1-Tak,0-Nie
squid_conf=Pełna ścieżka do pliku konfiguracyjnego Squida,0
squid_start=Polecenie uruchamiające Squida,3,Automatycznie
squid_stop=Polecenie zatrzymujące Squida,3,Automatycznie
squid_path=Program Squid,0
pid_file=Pełna ścieżka do pliku z&nbsp;numerem PID Squida,0
cache_dir=Pełna ścieżka do katalogu cache Squida,0
cachemgr_path=Program <tt>cachemgr.cgi</tt> Squida,0
log_dir=Pełna ścieżka do katalogu logów Squida,0
calamaris=Ścieżka do programu analizy logów <tt>calamaris</tt>,3,Nie zainstalowano
