line1=Opcje konfiguracyjne,11
show_status=Pokazywać stan włączenia i&nbsp;przyjmowania połączeń zamiast sterownika?,1,1-Tak,0-Nie
queue_refresh=Liczba sekund oczekiwania na odświeżenie kolejki drukowania,3,Nie odświeżaj
sort_mode=Sortuj drukarki według,1,1-Nazwy,0-Kolejności w systemie
display_mode=Na głównej stronie wyświetl,1,1-Tylko nazwy drukarek,0-Wszystkie szczegóły
show_jobs=Wyświetlać liczbę zadań w kolejce na stronie głównej?,1,1-Tak,0-Nie
line2=Konfiguracja systemu,11
print_style=Styl konfiguracji drukarek,1,solaris-Solaris,linux-Linux,freebsd-FreeBSD,hpux-HPUX,lprng-LPRng,unixware-UnixWare,irix-IRIX
driver_style=Typ sterowników drukarek,1,webmin-Webmin,caldera-Caldera,redhat-Redhat,hpux-HPUX,suse-SuSE,irix-IRIX
printcap_file=Plik opisu drukarek,3,Brak
ro_printcap_file=Dodatkowe pliki drukarek nie zarządzanych przez Webmina,3,Brak
spool_dir=Katalog zadań drukarki,3,Brak,60
model_path=Katalog zawierający filtry programowe,3,Brak
smbclient_path=Ścieżka do programu smbclient,3,Brak
gs_path=Ścieżka do programu ghostscript,3,Brak
gs_fontpath=Katalogi czcionek ghostscripta,3,Domyślne
gs_lib=Katalogi bibliotek Ghostscripta,3,Domyślne
hpnp_path=Ścieżka do programu hpnp,3,Brak
start_cmd=Polecenie uruchamiające zarządcę kolejek,3,Określone przez wzorzec drukarki
stop_cmd=Polecenie zatrzymujące zarządcę kolejek,3,Określone przez wzorzec drukarki
apply_cmd=Polecenie do uruchomienia po utworzeniu dowolnej zmiany,3,Brak
