line1=Opcje konfiguracyjne,11
lines=Domyślna liczba wyświetlanych linii,0
refresh=Sekundy pomiędzy odświeżeniami loga,3,Nigdy
compressed=Dołączać skompresowane logi do wyszukiwarki?,1,1-Tak,0-Nie
others=Wyświetlić logi z innych modułów?,1,1-Tak,0-Nie
extras=Inne pliki logów do wyświetlenia<br>(Z opcjonalnymi opisami),9,50,4,\t
line2=Konfiguracja systemu,11
syslog_conf=Ścieżka do pliku konfiguracyjnego sysloga,0
m4_conf=Czy konfiguracja sysloga tworzona poprzez m4?,1,1-Tak,0-Nie
m4_path=Ścieżka do m4,0
sync=Można wyłączyć synchronizację po każdym komunikacie?,1,1-Tak,0-Nie
pipe=Może logować do nazwanych potoków?,1,2-Do poleceń,1-Do nazwanych potoków,0-Nie
socket=Obsługa logowania do socketów?,1,1-Tak,0-Nie
pri_dir=Obsługa zaawansowanego wyboru priorytetów?,1,2-Styl FreeBSD,1-Styl Linuksa,0-Nie
pri_all=Obsługuje wszystkie priorytety?,1,1-Tak,0-Nie
tags=Obsługa oznaczonych sekcji?,1,1-Tak,0-Nie
pid_file=Plik(i) z numerem PID sysloga,3,Brak
syslogd=Ścieżka do programu serwera syslog,0
facilities=Obsługiwane pochodzenie zdarzenia,0
start_cmd=Polecenie do uruchomienia syslog,3,Tylko uruchom serwer
restart_cmd=Polecenie do zastosowania zmian,3,Zatrzymaj i ponownie uruchom serwer
signal_cmd=Polecenie do ponownego otwarcia plików logów,3,Wyślij sygnał HUP
tail_cmd=Polecenie Taiil i opcje,3,Domyślnie (tail -n LINES)
