line1=Možnosti konfigurace,11
login=Jméno administrátora MySQL,0
pass=Heslo administrátora MySQL,0
perpage=Počet řádků zobrazených na stránce,0
style=Ukázat databáze a tabulky jako,1,1-seznam,0-Ikony
add_mode=Použít rozhraní s editací řádku vertikálně?,1,1-ano,0-ne
blob_mode=Zobrazit pole blob a text jako,1,0-Data v tabulce,1-Odkazy ke stažení
nodbi=Použít DBI připojení (je-li k dispozici)?,1,0-ano,1-ne
date_subs=Uplatnit <tt>strftime</tt> substituce v cíli zálohování?,1,1-Ano,0-Ne
passwd_mode=Mod pro transformaci hesla,1,1-Starý,0-Výchozí
encoding=Kódování pro obsah databáze,3,Výchozí (z daného jazyka)
max_dbs=Maximální počet zobrazených databází a tabulek,0,5
max_text=Maximální zobrazovaná délka pro textová pole,3,Neomazeno
charset=Znaková sada pro MySQL data,15,charset
line2=Konfigurace systému,11
mysqlshow=Cesta k příkazu mysqlshow,0
mysqladmin=Cesta k příkazu mysqladmin,0
mysql=Cesta k příkazu mysql,0
mysqldump=Cesta k příkazu mysqldump,0
mysqlimport=Cesta k příkazu mysqlimport,0
mysqld=Cesta k příkazu mysqld,3,Automaticky
start_cmd=Příkaz ke startu mysql serveru,0
stop_cmd=Příkaz k zastavení mysql serveru,3,Automaticky
mysql_libs=Cesta k adresáři sdílených knihoven,3,Nic
mysql_data=Cesta k adresáři s MySQL databázemi,3,Nic
host=MySQL host pro připojení k,3,localhost
port=MySQL port pro připojení k,3,Výchozí
sock=MySQL socket soubor,3,Výchozí
my_cnf=Konfigurační soubor MySQL,0
