line1=Nastavení konfigurace,11
browse_server=Požadovat seznam sdílení od serveru,3,localhost
long_fstypes=Zobrazit dlouhá jména u souborových systémů?,1,1-Ano,0-Ne
sort_mode=Třídit souborové systémy podle,1,2-Bodu připojení,1-Typu,0-Pořadí v souborech
show_used=Zobrazit použité místo na disku v seznamu souborových systémů,1,1-Ano,0-Ne
delete_under=Při odpojování smazat adresář&#44; pokud je podardesářem,3,Nemazat nikdy
line2=Konfigurace systému,11
fstab_file=Soubor se souborovými systémy připojovanými při startu,0
auto_file=Soubor automatického připojování NFS,3
autofs_file=Jádro automaticky připojuje soubor,3
smbclient_path=Úplná cesta k smbclient,3
nmblookup_path=Úplná cesta k nmblookup,3
