line1=Možnosti konfigurace,11
show_status=Zobrazit použitelné a akceptovatelné stavy místo driveru?,1,1-ano,0-ne
queue_refresh=Počet vteřin před obnovou tiskové fronty,3,Neobnovovat
sort_mode=Seřadit tiskárny podle,1,1-jména,0-pořadí v systému
display_mode=Na hlavní stránce zobrazit,1,1-jenom jména tiskáren,0-Úplné detaily
show_jobs=Zobrazit na hlavní stránce počet úloh ve frontách?,1,1-ano,0-ne
line2=Konfigurace systému,11
print_style=Styl konfigurace tiskáren,4,solaris-Solaris,linux-Linux,freebsd-FreeBSD,hpux-HPUX,lprng-LPRng,unixware-UnixWare,cups-CUPS,irix-IRIX
driver_style=Styl driverů diskáren,4,webmin-Webmin,caldera-Caldera,redhat-Redhat,hpux-HPUX,suse-SuSE,cups-CUPS,irix-IRIX,printconf-Redhat Printconf
printcap_file=Soubor tiskáren,3,Nic
model_path=Adresář obsahující programy rozhraní,3,Nic
smbclient_path=Cesta k smbclient,3,Nic
gs_path=Cesta k ghostscript,3,Nic
gs_fontpath=Adresáře pro Ghostscript fonty,3,Výchozí
gs_lib=Adresáře Ghostscript knihoven,3,Výchozí
hpnp_path=Cesta k programu hpnp,3,Nic
start_cmd=Příkaz spouštějící plánovač,3,Determinován stylem tiskárny
stop_cmd=Příkaz ukončující plánovač,3,Determinován stylem tiskárny
apply_cmd=Příkaz spuštěný po jakékoliv změně,3,Nic
