line1=Možnosti konfigurace,11
lease_sort=Třídit pronájmy podle,1,0-pořadí v souboru,1-IP adresy,2-jména hosta
hostnet_list=Zobrazit podsítě a hosty jako,1,0-ikony,1-seznam
dhcpd_nocols=Ikony v řádku,0
lease_tz=Zobrazit dobu pronájmu v,1,0-GMT,1-Lokálním čase
show_ip=Zobrazit IP adresy pro hosty?,1,1-ano,0-ne
show_mac=Zobrazit MAC adresy hostů?,1,1-Ano,0-Ne
group_name=Zobrazení jmen skupin jako,1,1-<tt>doménová jména</tt>,0-Jména nebo počtu členů,2-Popisu
display_max=Počet maximálně zobrazených podsítí a hostů,3,Bezomezení
line2=Konfigurace systému,11
dhcpd_conf=Konfigurační soubor DHCP serveru,0
dhcpd_path=Spouštěč DHCP serveru,0
start_cmd=Příkaz pro spuštění DHCP serveru,3,Spustit spouštěč server
restart_cmd=Příkaz pro zapsání konfigurace,3,Zabít a restartovat
stop_cmd=Příkaz pro zastavení DHCP serveru,3,Kill process
pid_file=Cesta k PID souboru DHCP serveru,0
lease_file=Soubor s pronájmy DHCP serveru,0
interfaces_type=Typ souboru s rozhraními,4,redhat-Redhat,mandrake-Mandrake,suse-SuSE,debian-Debian,caldera-Caldera,-Webmin
version=Verze DHCP serveru,3,Nastav automaticky
