line2=Konfiguracja systemu,11
man_cmd=Polecenie wyświetlające stronę podręcznika,0
man_cmd_sect=Polecenie wyświetlające stronę podręcznika z&nbsp;rozdziału,0
search_cmd=Polecenie przeszukujące strony,0
man_dir=Katalogi zawierające strony podręcznika (MANPATH),0
list_cmd=Polecenie zwracające scieżkę do pliku strony podręcznika,0
list_cmd_sect=Polecenie zwracające scieżkę do pliku strony podręcznika z&nbsp;rozdziału,0
man2html_path=Ścieżka do <tt>man2html</tt>,3,Nie korzystaj z&nbsp;<tt>man2html</tt>
doc_dir=Katalog dokumentacji pakietów,3,Brak
howto_dir=Katalog dokumentów "HOWTO",3,Brak
kernel_dir=Katalog dokumentacji kernela,3,Brak
kde_dir=Katalog dokumentacji KDE,3,Brak
custom_dir=Katalog własnej dokumentacji,3,Brak
custom_desc=Nazwa własnej dokumentacji,3,Domyślnie
