login=Login administracyjny,0
pass=Hasło administracyjne,0
perpage=Ilość wyświetlanych linii na stronie,0
style=Pokazywać bazy danych i&nbsp;tabele jako,1,1-Listę,0-Ikony
add_mode=Korzystać z&nbsp;interfejsu pionowej edycji wierszy,1,1-Tak,0-Nie
mysqlshow=Ścieżka do polecenia <tt>mysqlshow</tt>,0
mysqladmin=Ścieżka do polecenia <tt>mysqladmin</tt>,0
mysql=Ścieżka do polecenia <tt>mysql</tt>,0
mysqldump=Ścieżka do polecenia <tt>mysqldump</tt>,0
mysqlimport=Ścieżka do polecenia <tt>mysqlimport</tt>,0
start_cmd=Polecenie uruchamiające serwer MySQL,0
stop_cmd=Polecenie zatrzymujące serwer MySQL,3,Automatycznie
mysql_libs=Ścieżka do katalogu bibliotek współdzielonych MySQL,3,Brak
host=Host MySQL-a&#44; z&nbsp;którym się łączyć,3,Localhost
sock=Plik gniazd MySQLa,3,Domyślny
