line1=Nastavitelné volby,11
display_max=Maximální počet zobrazených uživatelů a skupin,0
threshold_pc=Procentní hranice pro vysvícení použité kvóty,3,Nezobrazit
pc_show=Použít procenta pro zobrazení,1,2-Těžká a pevná,1-Pouze těžká,0-Pouze lehká
sort_mode=Setřídit uživatele a skupiny podle,1,0-Použitých bloků,2-Jména,1-pořadí z  repquota,3-Těžké kvóty podle bloků,4-Lehké kvóty podle bloků,5-Těžké kvóty podle procent,6-Lehké kvóty podle procent
block_mode=Zobrazit kvóty v,1,1-Kilobytech (kde je to možné),0-Blocích
email_msg=Emailová zpráva pro uživatele&#44; kteří mají přesahující kvótu,9,80,5,\t
email_subject=Předmět pro emailovou zprávu,3,Výchozí
line2=Konfigurace systému,11
show_grace=Zobraz časy milosti&#44; je&#45;li to možné?,1,1-Ano,0-Ne
user_repquota_command=Příkaz pro zobrazení seznamu uživatelů v systému,0
group_repquota_command=Příkaz pro zobrazení seznamu skupin v systému,0
user_edquota_command=Příkaz pro úpravu kvóty uživatele,0
group_edquota_command=Příkaz pro úpravu kvóty skupiny,0
user_setquota_command=Příkaz pro nastavení kvót pro uživatele,3,Nic
group_setquota_command=Příkaz pro nastavení kvót pro skupiny,3,Nic
user_quota_command=Příkaz pro kontrolu kvóty uživatele,0
group_quota_command=Příkaz pro kontrolu kvóty skupiny,0
user_copy_command=Příkaz pro kopírování kvóty uživatele,0
group_copy_command=Příkaz pro kopírování kvóty skupiny,0
user_quotaon_command=Příkaz pro zapnutí kvót pro uživatele,0
group_quotaon_command=Příkaz pro zapnutí kvót pro skupiny,0
user_quotaoff_command=Příkaz pro vypnutí kvót pro uživatele,0
group_quotaoff_command=Příkaz pro vypnutí kvót pro skupiny,0
quotacheck_command=Příkaz pro kontrolu kvót,0
user_grace_command=Příkaz pro úpravu času milosti u uživatele,0
group_grace_command=Příkaz pro úpravu času milosti u skupiny,0
