local_cf=SpamAssassin αρχείο ή φάκελος ρυθμίσεων,0
spamassassin=Πλήρης διαδρομή στην εντολή SpamAssassin,0
sa_learn=Πλήρης διαδρομή στην εντολή sa-learn,0
warn_procmail=Εμφάνιση προειδοποίησης εάν το SpamAssassin δεν είναι ρυθμισμένο;,1,1-Ναι,0-Όχι
procmailrc=Procmail αρχείο ρυθμίσεων,3,Από το άρθρωμα Procmail
processes=SpamAssassin ονόματα daemon διεργασιών,0
restart_cmd=Εντολή επανεκκίνησης διεργασιών,3,Απλά στείλε σήμα HUP
procmail_cmd=SpamAssassin εντολή για ρύθμιση Procmail,10,*-Αυτόματη ανίχνευση
before_cmd=Εντολή προς εκτέλεση πριν αλλάξουν οι κανόνες,3,Καμία
after_cmd=Εντολή προς εκτέλεση αφού αλλάξουν οι κανόνες,3,Καμία
