line2=Konfiguracja systemu,11
sshd_path=Pełna ścieżka do programu <tt>sshd</tt>,0
sshd_config=Pełna ścieżka do pliku konfiguracyjnego sshd,0
client_config=Pełna ścieżka do pliku konfiguracyjnego klienta ssh,0
pid_file=Pełna ścieżka do pliku z&nbsp;numerem PID sshd,0
start_cmd=Polecenie uruchamiające sshd,3,Tylko uruchom serwer
restart_cmd=Polecenie do zastosowania konfiguracji SSHD,3,Tylko wyślij sygnał HUP
stop_cmd=Polecenie do zatrzymania sshd,3,Tylko zabij proces
keygen_path=Pełna ścieżka do programu ssh-keygen,0
sshd_version=Wersja serwera SSH,3,Wykryj automatycznie
