line0=Ścieżki BINDa,11
chroot=Katalog chroot dla BINDa,3,Domyślny
no_chroot=Czy named.conf znajduje się w katalogu chroot?,1,0-Tak,1-Nie
auto_chroot=Komenda do wyszukania katalogu chroot,3,Użyj powyższego katalogu
named_user=BIND jest uruchamiany jako użytkownik,3,Domyślny
named_group=BIND jest uruchamiany jako grupa,3,Domyślna
zones_file=Dodawać nowe strefy do pliku,3,named.conf
relative_paths=Użyć ścieżek pliku strefy względnej w <tt>named.conf</tt>?,1,1-Tak,0-Nie
keys_dir=Katalog dla kluczy DNSSEC,3,Taki sam jak pliki strefy
line1=Wyświetl opcje,11
show_list=Wyświetlaj domeny jako,1,0-Ikony,1-Listę
by_view=Kategoryzować strefy według widoku?,1,1-Tak,0-Nie
records_order=Kolejność wyświetlania rekordów,1,1-Wg nazwy,2-Wg wartości,3-Wg IP,0-W&nbsp;kolejności dodawania
max_zones=Maksymalna liczba wyświetlanych stref,0
rev_def=Aktualizacja DNS odwrotnego jest,1,0-Domyślnie włączona,1-Domyślnie wyłączona
rev_must=Strefa odwrotna musi istnieć?,1,1-Tak,0-Nie
support_aaaa=Włączyć obsługę DNS dla adresów IPv6,1,0-Nie,1-Tak
allow_comments=Zezwól na komentarze dla rekordów,1,0-Nie,1-Tak
allow_wild=Zezwolić na wieloznaczne,1,0-Nie,1-Tak
allow_underscore=Zezwolić na podkreślenia w nazwach rekordów?,1,1-Tak,0-Nie
short_names=Konwertować nazwy rekordów do formy kanonicznej,1,0-Tak,1-Nie
extra_forward=Dodatkowe rodzaje rekordu dla stref prostych,0
extra_reverse=Dodatkowe rodzaje rekordu dla stref odwrotnych,0
largezones=Wyświetlać rekordy w strefach?,1,0-Tak,1-Nie (dla dużych stref)
default_view=Domyślny widok dla nowych stref,3,Pierwszy na liście
line2=Opcje pliku strefy,11
soa_style=Postać numeru seryjnego,1,0-Numer kolejny,1-Oparty na dacie (RRRRMMDDnn),2-Czas unixowy
soa_start=Numer startowy w SOA bazującym na dacie,0,3
master_ttl=Dodawać $ttl na początku nowych plików stref,1,1-Tak,0-Nie
master_dir=Katalog plików stref podstawowych,3,Domyślny
slave_dir=Katalog plików stref zapasowych/wykarczowanych,3,Domyślny
forwardzonefilename_format=Format nazwy plików strefy prostej,0
reversezonefilename_format=Format dla nazwy pliku strefy odwrotnej,0
file_owner=Właściciel plików stref (użytkownik:grupa),3,Domyślny
file_perms=Uprawnienia dla plików stref (ósemkowo),3,Domyślne
default_prins=Domyślny serwer główny dla stref głównych,3,Nazwa hosta systemu
default_master=Domyślny serwer(y) dla stref zapasowych,0
updserial_on=Automatycznie uaktualniać numery seryjne,1,1-Tak,0-Nie
ipv6_mode=Domena dla adresów odwrotnych IPv6,1,1-iop6.arpa,0-ip6.int
confirm_zone=Potwierdzać usuwanie stref?,1,1-Tak,0-Nie
confirm_rec=Potwierdzać usuwanie rekordów?,1,1-Tak,0-Nie
free_nets=Sieci IP dla wolnych adresów,3,Automatycznie
force_random=Źródło Entropy dla kluczy DNSSEC,1,1-/dev/random (bezpieczne&#44; ale wolne),0-/dev/urandom (Prawdopodobnie niebezpieczne&#44; ale szybkie)
spf_record=Prawdziwy typ rekordu dla Sender Permitted From,1,1-SPF,0-TXT
line2.5=Klaster serwerów zapasowych,11
this_ip=IP domyślnego serwera głównego dla zdalnych stref zapasowych,3,Adres IP nazwy hosta
other_slaves=Dodać inne adresy IP serwerów zapasowych do listy serwerów głównych na serwerach zapasowych?,1,1-Tak,0-Nie
extra_slaves=Dodatkowe adresy IP serwera głównego dla stref zapasowych,3,Brak
line3=Konfiguracja systemu,11
named_conf=Pełna ścieżka do pliku named.conf,0
named_path=Pełna ścieżka do programu named,0
whois_cmd=Pełna ścieżka do polecenia whois,0
ndc_cmd=Pełna ścieżka do komendy ndc,0
rndc_cmd=Pełna ściezka do komendy rndc,0
rndcconf_cmd=Pełna ścieżka do komendy rndc-confgen,0
rndc_conf=Pełna ściezka do pliku rndc.conf,0
checkzone=Pełna ścieżka do komendy named-checkzone,0
checkconf=Pełna ścieżka do komendy named-chceckconf,0
keygen=Pełna ścieżka do komendy dnssec-keygen,0
signzone=Pełna ścieżka do komendy dnssec-signzone,0
pid_file=Domyślna lokalizacja pliku z&nbsp;numerem PID,3,/var/run/named.pid
no_pid_chroot=Plik PID jest w katalogu chroot?,1,0-Tak,1-Nie
start_cmd=Polecenie uruchamiające BINDa,3,Domyślne
stop_cmd=Komenda do zatrzymania BINDa,3,Zabij proces
restart_cmd=Komenda do zastosowania konfiguracji BINDa,10,-Wyślij sygnał HUP, restart-Zatrzymaj i zrestartuj,Inna komenda
dnssectools_conf=Pełna ścieżka do pliku dnssec-tools.conf,0
dnssectools_rollmgr_pidfile=Pełna ścieżka do plików dnssec-tools rollmgr pid,0
dnssectools_rollrec=Pełna ścieżka do plików dnssec-tools rollrec,0
dnssectools_keydir=Pełna ścieżka do katalogów dnssec-tools key,0
