line1=Nastavení konfigurace,11
dont_convert=Nepřidávat seznam Unix uživatelů do seznamu samba hesel,0
sort_mode=Třídit uživatele a skupiny podle jména?,1,1-ano,0-ne
line2=Konfigurace systému,11
smb_conf=Umístění konfiguračního souboru Samby,0
smb_passwd=Umístění souboru s hesly Samby,3
samba_status_program=Úplná cesta k smbstatus,0
samba_password_program=Úplná cesta k smbpasswd,0
samba_server=Úplná cesta k smbd,0
name_server=Úplná cesta k nmbd,0
swat_path=Úplná cesta k swat,3,Nic
smbgroupedit=Úplná cesta k smbgroupedit,3,Nic
pdbedit=Úplná cesta k pdbedit,3,Nic
net=Úplná cesta k příkazu net,3,Nic
start_cmd=Příkaz spouštějící Samba servery,3,Nic
stop_cmd=Příkaz ukončující Samba servery,3,Nic
