line1=Možnosti konfigurace,11
mailq_refresh=Časová prodleva (ve vteřinách) při obnovení seznamu zpráv,3,Neobnovovat
perpage=Počet zpráv zobrazených na stránce,0
wrap_width=Šířka k zalomení poštovních zpráv na,0
sort_mode=Seřazení tabulek podle,1,0-pořadí v souboru,1-jména
columns=Sloupce pro aliasy a jiné tabulky,1,2-2,1-1
show_cmts=Zobrazit v tabulkách popisy?,1,1-Ano,0-Ne
prefix_cmts=Zapsat předponu před všechny komentáře popisů textem <tt>Webmin</tt>?,1,1-Ano,0-Ne
send_mode=Poslat poštu v případě,3,Sendmail je spuštěn
max_records=Maximální počet záznamů zobrazených v tabulkách,0,6
mailq_show=Záhlaví zobrazená v seznamu zpráv,2,Date-Datum,From-Od koho,To-Komu,Subject-Předmět,Cc-Kopie,Size-Velikost,Status-Stav
mailq_sort=Třídit mailovou frontu podle,1,0-ID fronty,1-Adresa Od,2-Adresa Komu,3-Předmět,4-Status,5-Velikost
mailq_count=Zobrazit na hlavní stránce velikost mailové fronty?,1,0-Ano,1-Ne
mailq_order=Setřídit během mazání mailové fronty,1,-Výchozí,priorita-Priorita,host-Hostname,čas-Datum přijetí
line2=Konfigurace systému,11
sendmail_cf=Úpplná cesta k sendmail.cf,8
sendmail_features=Základní adresář sendmailu M4,7
sendmail_mc=Úplná cesta ke konfiguračnímu souboru M4,8
sendmail_pid=Úplná cesta k pid souboru sendmailu,8
sendmail_smf=Solaris SMF název služby,3,Nic (použít PID soubory)
sendmail_command=Spouštěcí příkaz serverového módu sendmailu,0
sendmail_stop_command=Příkaz k ukončení sendmail,3,Zabít proces
sendmail_restart_command=Příkaz restartující sendmail,3,Stop a start
makemap_path=Příkaz Makemap,0
sendmail_path=Příkaz Sendmail,0
alias_file=Úplná cesta k souboru aliases pro sendmail,3,Automatická
virtusers_file=Zdrojovový soubor pro databázi virtuálních uživatelů,3,Shodný s DBM
mailers_file=Zdrojový soubor pro databázi mailer tabulek,3,SShodný s DBM
generics_file=Zdrojový soubor pro databázi generics,3,Shodný s DBM
access_file=Zdrojový soubor pro databázi přístupů,3,Shodný s DBM
domains_file=Zdrojový soubor pro databázi domén,3,Shodný s DBM
mail_dir=Umístění souboru pro zprávy uživatele,3,Soubor v domovském adresáři
mail_file=Soubor zpráv v domovském adresáři,0
mail_type=Typ úschovny mailů,1,1-Maildir,0-mbox
mail_style=Styl adresářů v souboru zpráv,4,0-mail/username,1-mail/u/username,2-mail/u/us/username,3-mail/u/s/username
mail_sync=Synchronizace souboru mailu uživatele,2,vytvořit-Vytvořit s uživatelem,smazat-Smazat s uživatelem,změnit-Přejmenovat s uživatelem
smrsh_dir=SMRSH adresář,3,nic
queue_dirs=Speciální adresáře pro seznam zpráv,3,nic
