line1=Možnosti konfigurace,11
expert=Povolit výběr jednotlivých úrovní běhu?,1,1-ano,0-ne
desc=Zobrazit akce s popisy?,1,2-ano&#44; a zobrazit všechny úrovně běhu,1-ano,0-ne
order=Zobrazit pořadí bootování v akcích?,1,0-Ano,1-Ne
status_check=Zobrazit aktuální stav akcí,1,2-jako seznam více akcí,1-kdy jedna stránka náleží jedné akci,0-ne
sort_mode=Setřídit akce podle,1,1-pořadí při bootování,0-názvu
line2=Konfigurace systému,11
init_base=Adresář obsahující podadresáře úrovní běhu,0
init_dir=Adresář obsahující hlavní init skripty,0
order_digits=Počet cifer v pořadí akcí,0
boot_levels=Úrovně spuštění při vytvoření nové akce v,3,Z /etc/inittab
local_script=Skript lokálních spouštěcích příkazů,3
local_down=skript pro příkaz lokálního shození,3,Nic
reboot_command=Příkaz pro rebootování systému,0
shutdown_command=Příkaz pro shození systému,0
inittab_id=ID inittabu pro bootovací úroveň běhu,0
rc_dir=Adresáře pro FreeBSD rc skripty,3,Nic
rc_conf=Soubory FreeBSD konfigurace,3,Nic
line3=Konfigurace OSX systému,11
startup_dirs=Adresáře pro Darwin StartupItems,0
darwin_setup=Adresáře pro vlastní StartupItems,0
hostconfig=Darwin hostconfig soubor,0
plist=Název plistu v StartupItems adresáři,0
