max_users=Μέγιστος αριθμός χρηστών για προς εμφάνιση,0
input_type=Τύπος επιλογής χρηστών,1,1-Λίστα,0-Πεδίο κειμένου
sort_mode=Ταξινόμηση χρηστών κατά,1,0-Ταξινόμηση στο αρχείο,1-Κατά όνομα χρήστη
passwd_cmd=Εντολή αλλαγής κωδικού,3,Ενημέρωση αρχείων συστήματος απευθείας
