line0=Ρυθμίσιμες επιλογές,11
wrap_width=Πλάτος αναδίπλωσης μηνυμάτων email σε,0,6
perpage=Εμφάνιση μηνύματων email ανά σελίδα,0,6
show_to=Εμφάνιση διεύθυνσης Προς: στις γραμματοθυρίδες?,1,1-Ναι,0-Όχι
top_buttons=Εμφάνιση κουμπιών στο πάνω μέρος για,1,2-Γραμματοθυρίδες και emails,1-Γραμματοθυρίδες μόνο,0-Ποτέ
arrows=Εμφάνιση βελών αλλαγής σελίδας στο κάτω μέρος για,1,2-Γραμματοθυρίδες και emails,1-Γραμματοθυρίδες μόνο,0-Ποτέ
show_delall=Εμφάνιση κουμπιού διαγραφής όλης της γραμματοθυρίδας;,1,1-Ναι,0-Όχι
show_sent=Εμφάνιση αριθμού μηνυμάτων στο φάκελο απεσταλμένων;,1,1-Ναι,0-Όχι
fwd_mode=Προώθηση μηνυμάτων με εισαγωγικά σημεία στίξεως;,1,0-Ναι,1-Όχι
delete_warn=Να ζητείται επιβεβαίωση πριν από την διαγραφή;,10,y-Ναι,n-Όχι,Για αρχεία με μέγεθος πάνω από
view_html=Εμφανιση κειμένου μηνύματος ως,4,0-Πάντα απλό κείμενο,1-Κείμενο εάν είναι δυνατόν&#44; HTML αλλιώς,2-HTML αν είναι δυνατόν&#44; κείμενο αλλιώς,3-Μετατροπή HTML σε απλό κείμενο
html_edit=Χρήση επεξεργαστή HTML για σύνθεση;,1,2-Πάντα,1-Κατά την απάντηση σε HTML email,0-Ποτέ
html_quote=HTML τρόπος εμφάνισης εισαγωγικών,1,1-Το μήνυμα απο κάτω &lt;hr&gt;,0-Το μήνυμα μέσα &lt;blockquote&gt;
log_read=Καταγραφή της ανάγνωσης email στο αρχείο Καταγραφής Ενεργειών του Webmin;,1,1-Ναι,0-Όχι
bcc_to=Κρυφή Κοινοποίηση: αποστολή μηνυμάτων σε,0
sig_file=Αρχείο υπογραφής,10,*-Κανένα,.signature-~/.signature,Άλλο αρχείο
show_body=Εμφάνιση προεπισκοπίσεων σώματος μηνύματος στην λίστα;,1,1-Ναι,0-Όχι
open_mode=Άνοιγμα μηνυμάτων σε,1,1-Νέο παράθυρο,0-Στο παράθυρο της λίστας
link_mode=Άνοιγμα συνδέσμων σε,1,1-Νέο παράθυρο,0-Ίδιο παράθυρο
download=Συνημμένοι τύποι MIME για τους οποίους θα γίνεται πάντα λήψη,9,20,4,\t
date_fmt=Τρόπος εμφάνισης ημερομηνίας στο email list,1,auto-Locale based,dmy-DD/MM/YYYY,mdy-MM/DD/YYYY,ymd-YYYY/MM/DD
date_tz=Ζώνη ώρας για εμφάνιση ημερομηνιών,3,Προκαθορισμένη Συστήματος
max_records=Μέγιστος αριθμός χρηστών προς εμφάνιση,0,6
show_size=Τρόπος εμφάνισης χρηστών,1,0-Όνομα χρήστη μόνο,1-Όνομα χρήστη και μέγεθος,2-Πλήρεις λεπτομέριες
show_size_below=Όνομα χρήστη και μέγεθος - Που να εμφανίζεται το μέγεθος,1,0-Δεξιά από το όνομα χρήστη,1-Κάτω από το όνομα χρήστη
column_count=Αριθμός στηλών στις οποίες να εμφανίζονται τα ονόματα χρηστών,1,3-3,4-4,5-5,6-6,7-7,8-8,9-9
show_count=Εμφάνιση αριθμού μηνυμάτων στα εισερχόμενα;,1,1-Ναι,0-Όχι
sort_mode=Ταξινόμηση γραμματοθυρίδεν κατά,1,2-Μέγεθος,1-Όνομα χρήστη,0-Σειρά στο αρχείο password
show_mail=Να εμφανίζονται μόνο οι χρήστες που έχουν mail;,1,1-Ναι,0-Όχι
size_mode=Να συμπεριλαμβάνονται όλοι οι φακέλοι στο μέγεθος;,1,1-Ναι,0-Όχι (Ο πρώτος φάκελος μόνο)
ignore_users=Αγνόηση των παρακάτω ονομάτων χρηστών (να μην εμφανίζονται),15,userIgnoreList
ignore_users_enabled=Κατάσταση λίστας αγνόησης χρηστών,1,1-Ενεργοποιημένη,0-Απενεργοποιημένη
track_read=Σημείωση αναγνωσμένων/μη αναγνωσμένων emails,1,1-Ναι,0-Όχι
check_mod=Έλεγχος αλλαγών στην γραμματοθυρίδα κατά την διαγραφή email?,1,1-Ναι,0-Όχι
line3=Επιλογές Spam,11
spam_buttons=Εμφάνιση κουμπιών σημείωσης ως spam για,2,list-Γραμματοθυρίδες,mail-Μηνύματα
spam_del=Διαγραφή spam κατά την σημείωση;,1,1-Ναι,0-Όχι
spam_report=Σημείωση ως spam με χρήση,1,sa_learn-sa&#45;learn --spam,spamassassin-spamassasin -r,-Να αποφασίζεται αυτόματα
line3.5=Επιλογές διεύθυνσης Από,11
from_addr=Διεύθυνση Από: που θα χρησιμοποιείται όταν αποστέλλεται email χειρωκίνητα,3,Από το όνομα χρήστη γραμματοθυρίδας
webmin_from=Διεύθυνση Από: που θα χρησιμοποιείται όταν αποστέλλεται email από το Webmin,3,Προκαθορισμένη (webmin-noreply@<i>yourhost</i>)
from_virtualmin=Λήψη διέυθυνσης Από: από το Virtualmin?,1,1-Ναι,0-Όχι
from_dom=Domain που θα χρησιμοποιείται στην διεύθυνση Από:,3,hostname συστήματος
line1=Ρύθμιση συστήματος,11
mail_system=Εγκατεστημένος Διακομιστής Εmail,4,1-Sendmail,0-Postfix,2-Qmail,4-Qmail+LDAP,5-Qmail+VPopMail,3-Αυτόματος εντοπισμός
send_mode=Αποστολή email μέσω,10,-Εφαρμογής διακομιστή Mail,SMTP διακομιστή
no_crlf=Προσθήκη αλλαγής γραμμής ( \r ) σε κάθε γραμμή;,1,0-Ναι,1-Όχι
smtp_user=Όνομα σύνδεσης SMTP διακομιστή,3,Κανένα
smtp_pass=Κωδικός σύνδεσης SMTP διακομιστή,3,Κανένα
smtp_auth=Πιστοποίηση SMTP method,4,-Προκαθορισμένηt,Cram-MD5-Cram&#45;MD5,Digest-MD5-Digest&#45;MD5,Plain-Plain,Login-Login
auto=Ανίχνευση τοποθεσίας αρχείων email αυτόματα;,1,1-Ναι&#44; βάση του διακομιστη email,0-Όχι&#44; χρήση των παρακάτω ρυθμίσεων ..
mail_dir=Φάκελος email χρήστη,3,Κανένας
mail_style=Στύλ φακέλου email χρήστη,4,0-mail/username,1-mail/u/username,2-mail/u/us/username,3-mail/u/s/username
mail_file=Αρχείο email στον φάκελο χρήστη,3,Κανένα
mail_sub=Φάκελος email στον φάκελο χρήστη,3,Κανένας
mail_usermin=Usermin-style υποκατάλογος φακέλων στον φάκελο χρήστη,3,Κανένας
mailbox_user=Φάκελος ρυθμίσεων Ανάγνωσης Mail στον φάκελο χρήστη,3,Κανένας
line2=Συγχρονισμός χρήστη,11
sync_create=Δημιουργία γραμματοθυρίδας κατά την δημιουργία χρήστη;,1,1-Ναι,0-Όχι
sync_modify=Μετονομασία γραμματοθυρίδας όταν μετονομάζεται ο χρήστης;,1,1-Ναι,0-Όχι
sync_delete=Διαγραφή γραμματοθυρίδας όταν διαγράφεται ο χρήστης;,1,1-Ναι,0-Όχι
sync_perms=Δικαιώματα νέων γραμματοθυρίδων,0,4
line4=VPOPMail επιλογές,11
vpopmail_dir=Βασικός φάκελος για VPOPMail,0
line5=Qmail+LDAP επιλογές,11
ldap_host=LDAP διακομιστής,0
ldap_port=LDAP θύρα,3,Προκαθορισμένη
ldap_login=Σύνδεση για τον διακομιστή LDAP,0
ldap_pass=Κωδικός για τον διακομιστή LDAP,0
ldap_base=Base για χρήστες email,0
