line1=Opcje konfigurowalne,11
display_max=Maksymalna liczba wyświetlanych użytkowników lub grup,0
threshold_pc=Próg procentowy wykorzystanego limitu quota&#44; od którego podświetlać,3,Nie pokazuj
pc_show=Pokazuj procent wykorzystania limitu,1,2-Twardego i miękkiego,1-Tylko twardego,0-Tylko miękkiego
sort_mode=Sortuj użytkowników i grupy według,1,0-Bloków użytych,2-Nazwy,1-Kolejności z repquota,3-Twardego limitu bloków,4-Miękkiego limitu bloków,5-Procentu użycia z twardego limitu,6-Procentu użycia z miękkiego limitu
block_mode=Pokazuj quoty w ,1,1-Kilobajtach (jeśli możliwe),0-Blokach
hide_uids=Pokazywać usuniętych użytkowników?,1,0-Tak,1-Nie
line1.1=Wiadomości e-mail w sprawie quota,11
email_msg=Wiadomość do użytkowników przekraczających limit quoty,9,80,5,\t
email_subject=Temat wiadomości do użytkowników,3,Domyślnie
gemail_msg=Wiadomość do grup przekraczających limity quoty,9,80,5,\t
gemail_subject=Temat wiadomości do grup,3,Domyślnie
line2=Konfiguracja systemu,11
show_grace=Pokazywać czasy ulgi jeśli dostępne?,1,1-Tak,0-Nie
user_repquota_command=Polecenie wyświetlające użytkowników w&nbsp;systemie plików,0
group_repquota_command=Polecenie wyświetlające grupy w&nbsp;systemie plików,0
user_edquota_command=Polecenie do zmiany quota użytkownika,0
group_edquota_command=Polecenie do zmiany quota grupy,0
user_setquota_command=Polecenie do ustawienia quota użytkownika,3,Brak
group_setquota_command=Polecenie do ustawienia quota grupy,3,Brak
user_quota_command=Polecenie do sprawdzenia quota użytkownika,0
group_quota_command=Polecenie do sprawdzenia quota grupy,0
user_copy_command=Polecenie do skopiowania quota użytkownika,0
group_copy_command=Polecenie do skopiowania quota grupy,0
user_quotaon_command=Polecenie do włączenia quota użytkowników,0
group_quotaon_command=Polecenie do włączenia quota grup,0
user_quotaoff_command=Polecenie do wyłączenia quota użytkowników,0
group_quotaoff_command=Polecenie do wyłączenia quota grup,0
quotacheck_command=Polecenie do naliczenia quota,0
user_grace_command=Polecenie do zmiany czasu ulgi dla użytkowników,0
group_grace_command=Polecenie do zmiany czasu ulgi dla grup,0
