line1=Opcje konfiguracyjne,11
config_file=Modyfikowany plik konfiguracyjny fetchmaila,3,Pliki <tt>.fetchmailrc</tt> wszystkich użytkowników
view_mode=Wyświetl listę użytkowników jako,1,1-Tylko nazwy użytkowników,0-Nazwy użytkowników i konfiguracje fetchmail
max_users=Maksymalna liczba użytkowników do wyświetlenia,3,Bez limitu
daemon_user=Użytkownik&#44; jako który serwer fetchmail jest uruchamiany,0
mda_command=Polecenie dostarczanie e-maili,3,Użyj SMTP
line2=Konfiguracja systemu,11
fetchmail_path=Ścieżka do programu <tt>fetchmail</tt>,0
pid_file=Ścieżka do pliku daemona PID serwera fetchmail,0
start_cmd=Polecenie uruchamiania daemona fetchmail,3,Tylko uruchom fetchmail
stop_cmd=Polecenie zatrzymania daemona fetchmail,3,Tylko uruchom fetchmail -q
