line1=Opcje konfiguracyjne,11
dont_convert=Lista użytkowników Uniksa nie dodawanych do listy haseł Samby,0
sort_mode=Porządkować użytkowników i&nbsp;grupy wg nazw,1,1-Tak,0-Nie
line2=Konfiguracja systemu,11
smb_conf=Lokalizacja pliku konfiguracyjnego Samby,0
smb_passwd=Lokalizacja pliku haseł Samby,3
samba_status_program=Pełna ścieżka do <tt>smbstatus</tt>,0
samba_password_program=Pełna ścieżka do <tt>smbpasswd</tt>,0
samba_server=Pełna ścieżka do <tt>smbd</tt>,0
name_server=Pełna ścieżka do <tt>nmbd</tt>,0
winbind_server=Pełna ścieżka do <tt>winbindd</tt>,0
swat_path=Pełna ścieżka do <tt>swat</tt>,3,Brak
smbgroupedit=Pełna ścieżka do <tt>smbgroupedit</tt>,3,Brak
pdbedit=Pełna ścieżka do <tt>pdbedit</tt>,3,Brak
net=Pełna ścieżka do polecenia <tt>net</tt>,3,Brak
start_cmd=Polecenie uruchamiające serwery Samby,3,Brak
stop_cmd=Polecenie zatrzymujące serwery Samby,3,Brak
restart_cmd=Polecenie restartu serwerów Samba,3,Tylko zatrzymaj i uruchom
start_cmd_wb=Polecenie do uruchomienia serwerów Winbind,0
stop_cmd_wb=Polecenie do zatrzymania serwerów Winbind,0
