line1=Ρυθμίσιμες επιλογές,11
lines=Προκαθορισμένος αριθμός γραμμών προς εμφάνιση,0,6
refresh=Δευτερόλεπτα μεταξύ ανανεώσεων εμφάνισης αρχείων καταγραφής,3,Ποτέ
compressed=Συμπερίληψη συμπιεσμένων αρχείων καταγραφής στις αναζητήσεις;,1,1-Ναι,0-Όχι
others=Εμφάνιση αρχείων καταγραφής από άλλα αρθρώματας,1,1-Ναι,0-Όχι
extras=¶λλα αρχεία καταγραφής προς εμφάνιση<br>(Με προαιρετικές περιγραφές),9,50,4,\t
line2=Ρύθμιση συστήματος,11
syslog_conf=Διαδρομή αρχείου ρύθμισεων syslog,0
m4_conf=Η επεξεργασία τουα αρχείου ρυθμίσεων Syslog γίνεται μέσω m4;,1,1-Ναι,0-;Όχι
m4_path=Διαδρομή προς m4,0
sync=Ο συγχρονισμός μετά από κάθε μήνυμα να μπορεί να απενεργοποιηθεί;,1,1-Ναι,0-Όχι
pipe=Υποστήριξη καταγραφής σε σωληνώσεις;,1,2-Στις εντολές,1-Στις σωληνώσεις με ονόματα,0-Όχι
socket=Υποστήριξη καταγραφής σε υποδοχές;,1,1-Ναι,0-Όχι
pri_dir=Υποστήριξη προηγμένης επιλογής προτεραιότητας;,1,2-FreeBSD style,1-Linux style,0-Όχι
pri_all=Υποστήριξη όλων των προτεραιοτήτων;,1,1-Ναι,0-Όχι
tags=Υποστήριξη τομέων με ετικέτες;,1,1-Ναι,0-Όχι
pid_file=Syslog αρχείο PID,3,Κανένα
syslogd=Διαδρομή διακομιστή syslog,0
facilities=Υποστηριζόμενες δυνατότητες,0
start_cmd=Εντολή εκκίνησης syslog,3,Απλά εκτέλεση του διακομιστή
restart_cmd=Εντολή εφαρμογής αλλαών,3,Τερματισμός (Kill) και επανεκκίνηση διακομιστή
tail_cmd=Εντολή tail και παραμέτροι,3,Προκαθορισμένη (tail -n LINES)
