line1=Opcje konfiguracyjne,11
lease_sort=Uporządkuj dzierżawy wg,1,0-Kolejności w pliku,1-Adresów IP,2-Nazw hostów
hostnet_list=Pokaż podsieci i hosty jako,1,0-Ikony,1-Listę
dhcpd_nocols=Ilość ikon w wierszu,0
lease_tz=Pokazuj czas dzierżawy wg,1,0-GMT,1-Czasu lokalnego
lease_refresh=Sekund pomiędzy odświeżaniem listy dzierżawy,3,Nigdy
show_ip=Wyświetlać adres IP hostów?,1,1-Tak,0-Nie
show_mac=Wyświetlać adres MAC hostów?,1,1-Tak,0-Nie
group_name=Wyświetl nazwy grupy jako,1,1-<tt>domain&#45;name</tt>,0-Ilość lub nazwa użytkowników,2-Opis
display_max=Maksymalna liczba wyświetlanych podsieci i hostów,3,Nieograniczona
add_file=Dodaj nowe podsieci&#44; hosty i grupy do pliku,3,Główny plik konfiguracyjny
line2=Opcje systemowe,11
dhcpd_conf=Plik konfiguracyjny serwera DHCP,0
dhcpd_path=Plik programu serwera DHCP,0
start_cmd=Polecenie uruchomienia serwera DHCP,3,Wykonaj uruchomienie serwera
restart_cmd=Polecenia do zastosowania konfiguracji,3,Ubij i zrestartuj
stop_cmd=Polecenie zatrzymania serwera DHCP,3,Ubij proces
pid_file=Ścieżka do pliku z numerem PID serwera DHCP,0
lease_file=Plik dzierżaw serwera DHCP,0
interfaces_type=Typ interfejsów pliku,4,redhat-Redhat,mandrake-Mandrake,suse-SuSE,debian-Debian,caldera-Caldera,gentoo-Gentoo,-Webmin
version=Wersja serwera DHCP,3,Automatycznie
