line1=Nastavitelné volby,11
max_len=Maximální zobrazená délka příkazu,3,Neomezeno
max_jobs=Maximum úloh Cronu k zobrazení,3,Neomezeno
show_time=Zobrazit plány úlohy?,1,1-Ano,0-Ne
show_comment=Zobrazit komentáře úloh?1,1-Ano,0-Ne
show_run=Zobrazit stav běhu úlohy?,1,2-Ano a umožnit spuštění a zastavení,1-Ano,0-Ne
match_mode=Najít procesy úlohy podle,1,1-Příkazu,0-Příkazu a argumentů
match_user=Zohlednit uživatele při hledání procesu úlohy?,1,1-Ano,0-Ne
kill_subs=Zabít všechny pod-procesy při přerušení úlohy?,1,1-ano,0-Ne
hourly_only=Omezit úlohy na nejméně hodinový interval?,1,0-Ne,1-Ano
add_file=Přidat nové úlohy do souboru,3,Uživatelův crontab soubor
line2=Konfigurace systému,11
cron_dir=Crontab Adresář,0
cron_get_command=Příkaz pro čtení uživatelských úloh,0
cron_edit_command=Příkaz pro editaci uživatelských úloh,0
cron_copy_command=Příkaz pro akceptování uživatelských úloh na standardním vstupu,0
cron_delete_command=Program pro vymazání uživatelských úloh,0
cron_input=Cron podporuje stdin u plánovaných úloh,1,1-Ano,0-Ne
cron_allow_file=Seznam povolených uživatelů,0
cron_deny_file=Seznam zakázaných uživatelů,0
cron_deny_all=Přístup bez seznamů povolených nebo zakázaných uživatelů,1,0-Zakázat vše,1-Zakázat vše mimo roota,2-Povolit vše
vixie_cron=Podpora Vixie-Cron extensions,1,1-Ano,0-Ne
system_crontab=Cesta k Vixie-Cron system crontab souboru,0
single_file=Cesta k uživatelovu crontab souboru,0
cronfiles_dir=Cesta k cron.d adresáři,3,Nic
run_parts=spustitelná část příkazu,0
