login=Login administracyjny,0
pass=Hasło administracyjne,0
perpage=Ilość linii wyświetlanych na stronie,0
psql=Ścieżka do polecenia <tt>psql</tt>,0
plib=Ścieżka do bibliotek współdzielonych PostgreSQLa,3,Nie wymagana
basedb=Początkowa baza danych PostgreSQLa,0
start_cmd=Polecenie uruchamiające PostgreSQLa,0
stop_cmd=Polecenie zatrzymujące PostgreSQLa,3,Zabicie procesu
pid_file=Ścieżka z&nbsp;numerem PID procesu <tt>postmaster</tt>,0
host=Host PostgreSQLa&#44; z&nbsp;którym się łączyć,3,Localhost
